@@ -18,6 +18,9 @@ mod sealed {
 use arrayvec::ArrayVec;
 
 /// Defines a trait that chooses between libstd or libm implementations of float methods.
+///
+/// Some methods will eventually became available in core by
+/// [`core_float_math`](https://github.com/rust-lang/rust/issues/137578)
 macro_rules! define_float_funcs {
     ($(
         fn $name:ident(self $(,$arg:ident: $arg_ty:ty)*) -> $ret:ty
@@ -31,9 +34,16 @@ macro_rules! define_float_funcs {
         /// For documentation see the respective functions in the std library.
         #[cfg(not(feature = "std"))]
         pub trait FloatFuncs : Sized + sealed::FloatFuncsSealed {
-            /// Special implementation for signum, because libm doesn't have it.
+            /// For documentation see <https://doc.rust-lang.org/std/primitive.f64.html#method.signum>
+            ///
+            /// Special implementation, because libm doesn't have it.
             fn signum(self) -> Self;
 
+            /// For documentation see <https://doc.rust-lang.org/std/primitive.f64.html#method.rem_euclid>
+            ///
+            /// Special implementation, because libm doesn't have it.
+            fn rem_euclid(self, rhs: Self) -> Self;
+
             $(fn $name(self $(,$arg: $arg_ty)*) -> $ret;)+
         }
 
@@ -51,6 +61,16 @@ macro_rules! define_float_funcs {
                 }
             }
 
+            #[inline]
+            fn rem_euclid(self, rhs: Self) -> Self {
+                let r = self % rhs;
+                if r < 0.0 {
+                    r + rhs.abs()
+                } else {
+                    r
+                }
+            }
+
             $(fn $name(self $(,$arg: $arg_ty)*) -> $ret {
                 #[cfg(feature = "libm")]
                 return libm::$lfname(self $(,$arg as _)*);
@@ -73,6 +93,16 @@ macro_rules! define_float_funcs {
                 }
             }
 
+            #[inline]
+            fn rem_euclid(self, rhs: Self) -> Self {
+                let r = self % rhs;
+                if r < 0.0 {
+                    r + rhs.abs()
+                } else {
+                    r
+                }
+            }
+
             $(fn $name(self $(,$arg: $arg_ty)*) -> $ret {
                 #[cfg(feature = "libm")]
                 return libm::$lname(self $(,$arg as _)*);
@@ -310,7 +340,7 @@ pub fn solve_quartic(c0: f64, c1: f64, c2: f64, c3: f64, c4: f64) -> ArrayVec<f6
     }
     // Overflow happened, just return no roots.
     //println!("overflow, no roots returned");
-    Default::default()
+    ArrayVec::default()
 }
 
 fn solve_quartic_inner(a: f64, b: f64, c: f64, d: f64, rescale: bool) -> Option<ArrayVec<f64, 4>> {
