@@ -1,16 +1,11 @@
-//! The cpuid binary accompanying the library.
-//!
-//! The cpuid binary only compiles/runs on x86 platforms.
 use std::str::FromStr;
 
-use clap::{Parser, ValueEnum};
-use raw_cpuid::{CpuId, CpuIdReaderNative};
+use clap::Parser;
+use raw_cpuid::CpuId;
 
-#[derive(ValueEnum, Clone)]
 enum OutputFormat {
-    #[value(alias("raw"))]
     Raw,
-    #[value(alias("cli"))]
+    Json,
     Cli,
 }
 
@@ -20,6 +15,7 @@ impl FromStr for OutputFormat {
     fn from_str(s: &str) -> Result<Self, Self::Err> {
         match s {
             "raw" => Ok(OutputFormat::Raw),
+            "json" => Ok(OutputFormat::Json),
             "cli" => Ok(OutputFormat::Cli),
             _ => Err("no match"),
         }
@@ -32,14 +28,18 @@ impl FromStr for OutputFormat {
 #[clap(disable_colored_help(true))]
 struct Opts {
     /// Configures the output format.
-    #[clap(short, long, default_value = "cli")]
+    #[clap(short, long, default_value = "cli", possible_values = &["raw", "json", "cli", ])]
     format: OutputFormat,
 }
 
 fn main() {
     let opts: Opts = Opts::parse();
     match opts.format {
-        OutputFormat::Raw => raw_cpuid::display::raw(CpuIdReaderNative),
+        OutputFormat::Raw => raw_cpuid::display::raw(),
+        OutputFormat::Json => {
+            let cpuid = CpuId::new();
+            raw_cpuid::display::json(cpuid);
+        }
         OutputFormat::Cli => {
             let cpuid = CpuId::new();
             raw_cpuid::display::markdown(cpuid);
