@@ -7,6 +7,61 @@ use std::str;
 pub const IFT_TAG: types::Tag = Tag::new(b"IFT ");
 pub const IFTX_TAG: types::Tag = Tag::new(b"IFTX");
 
+/// Wrapper for the packed childEntryMatchModeAndCount field in IFT format 2 mapping table.
+///
+/// Reference: <https://w3c.github.io/IFT/Overview.html#mapping-entry-childentrymatchmodeandcount>
+///
+/// The MSB is a flag which indicates conjunctive (bit set) or disjunctive (bit cleared) matching.
+/// The remaining 7 bits are a count.
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq, PartialOrd, Ord, Hash)]
+pub struct MatchModeAndCount(u8);
+
+impl MatchModeAndCount {
+    /// Flag indicating that copy mode is append.
+    ///
+    /// See: <https://w3c.github.io/IFT/Overview.html#mapping-entry-copymodeandcount>
+    pub const MATCH_MODE_MASK: u8 = 0b10000000;
+
+    /// Mask for the low 7 bits to give the copy index count.
+    pub const COUNT_MASK: u8 = 0b01111111;
+
+    pub fn bits(self) -> u8 {
+        self.0
+    }
+
+    pub fn from_bits(bits: u8) -> Self {
+        Self(bits)
+    }
+
+    /// If true matching mode is conjunctive (... AND ...) otherwise disjunctive (... OR ...)
+    pub fn conjunctive_match(self) -> bool {
+        (self.0 & Self::MATCH_MODE_MASK) != 0
+    }
+
+    pub fn count(self) -> u8 {
+        self.0 & Self::COUNT_MASK
+    }
+}
+
+impl TryFrom<MatchModeAndCount> for usize {
+    type Error = ReadError;
+
+    fn try_from(value: MatchModeAndCount) -> Result<Self, Self::Error> {
+        Ok(value.count() as usize)
+    }
+}
+
+impl types::Scalar for MatchModeAndCount {
+    type Raw = <u8 as types::Scalar>::Raw;
+    fn to_raw(self) -> Self::Raw {
+        self.0.to_raw()
+    }
+    fn from_raw(raw: Self::Raw) -> Self {
+        let t = <u8>::from_raw(raw);
+        Self(t)
+    }
+}
+
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 #[derive(Clone, Debug, PartialEq, Eq, Default, Ord, PartialOrd, Hash)]
 pub struct CompatibilityId([u8; 16]);
@@ -111,45 +166,6 @@ impl U16Or24 {
     }
 }
 
-#[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct IdDeltaOrLength(i32);
-
-impl ReadArgs for IdDeltaOrLength {
-    type Args = Offset32;
-}
-
-impl ComputeSize for IdDeltaOrLength {
-    fn compute_size(entry_id_string_data_offset: &Offset32) -> Result<usize, ReadError> {
-        // This field is either a u16 or an int24 depending on whether or not string data
-        // is present. See: <https://w3c.github.io/IFT/Overview.html#mapping-entry-entryiddelta>
-        Ok(if entry_id_string_data_offset.is_null() {
-            3
-        } else {
-            2
-        })
-    }
-}
-
-impl FontReadWithArgs<'_> for IdDeltaOrLength {
-    fn read_with_args(
-        data: FontData<'_>,
-        entry_id_string_data_offset: &Self::Args,
-    ) -> Result<Self, ReadError> {
-        if entry_id_string_data_offset.is_null() {
-            data.read_at::<Int24>(0).map(|v| Self(i32::from(v)))
-        } else {
-            data.read_at::<u16>(0).map(|v| Self(v as i32))
-        }
-    }
-}
-
-impl IdDeltaOrLength {
-    #[inline]
-    pub fn into_inner(self) -> i32 {
-        self.0
-    }
-}
-
 impl<'a> PatchMapFormat1<'a> {
     pub fn gid_to_entry_iter(&'a self) -> impl Iterator<Item = (GlyphId, u16)> + 'a {
         GidToEntryIter {
@@ -167,11 +183,6 @@ impl<'a> PatchMapFormat1<'a> {
         self.max_entry_index() as u32 + 1
     }
 
-    pub fn uri_template_as_string(&self) -> Result<&str, ReadError> {
-        str::from_utf8(self.uri_template())
-            .map_err(|_| ReadError::MalformedData("Invalid UTF8 encoding for uri template."))
-    }
-
     pub fn is_entry_applied(&self, entry_index: u16) -> bool {
         let byte_index = entry_index / 8;
         let bit_mask = 1 << (entry_index % 8);
@@ -182,13 +193,6 @@ impl<'a> PatchMapFormat1<'a> {
     }
 }
 
-impl PatchMapFormat2<'_> {
-    pub fn uri_template_as_string(&self) -> Result<&str, ReadError> {
-        str::from_utf8(self.uri_template())
-            .map_err(|_| ReadError::MalformedData("Invalid UTF8 encoding for uri template."))
-    }
-}
-
 impl FeatureMap<'_> {
     pub fn entry_records_size(&self, max_entry_index: u16) -> Result<usize, ReadError> {
         let field_width = if max_entry_index < 256 { 1 } else { 2 };
@@ -338,7 +342,7 @@ mod tests {
     // - Test enforced minimum entry count of > 0.
     // - Test where entryIndex is a u16.
     // - Invalid table (too short).
-    // - Invalid UTF8 sequence in uri template.
+    // - Invalid UTF8 sequence in url template.
     // - Compat ID is to short.
     // - invalid entry map array (too short)
     // - feature map with short entry indices.
@@ -408,6 +412,84 @@ mod tests {
         assert_eq!(*fr1.entry_map_count(), U8Or16(0x02));
     }
 
+    #[test]
+    fn format_1_get_charstrings_offset() {
+        // No offsets
+        let data = test_data::simple_format1();
+        let table = Ift::read(FontData::new(&data)).unwrap();
+        let Ift::Format1(map) = table else {
+            panic!("Not format 1.");
+        };
+
+        assert_eq!(map.cff_charstrings_offset(), None);
+        assert_eq!(map.cff2_charstrings_offset(), None);
+
+        // One offset
+        let data = test_data::simple_format1_with_one_charstrings_offset();
+        let table = Ift::read(FontData::new(&data)).unwrap();
+        let Ift::Format1(map) = table else {
+            panic!("Not format 1.");
+        };
+
+        assert_eq!(map.cff_charstrings_offset(), Some(456));
+        assert_eq!(map.cff2_charstrings_offset(), None);
+
+        // Two offsets
+        let data = test_data::simple_format1_with_two_charstrings_offsets();
+        let table = Ift::read(FontData::new(&data)).unwrap();
+        let Ift::Format1(map) = table else {
+            panic!("Not format 1.");
+        };
+
+        assert_eq!(map.cff_charstrings_offset(), Some(456));
+        assert_eq!(map.cff2_charstrings_offset(), Some(789));
+    }
+
+    #[test]
+    fn format_2_get_charstrings_offset() {
+        // No offsets
+        let data = test_data::codepoints_only_format2();
+        let table = Ift::read(FontData::new(&data)).unwrap();
+        let Ift::Format2(map) = table else {
+            panic!("Not format 2.");
+        };
+        assert_eq!(map.cff_charstrings_offset(), None);
+        assert_eq!(map.cff2_charstrings_offset(), None);
+
+        // One offset
+        let data = test_data::format2_with_one_charstrings_offset();
+        let table = Ift::read(FontData::new(&data)).unwrap();
+        let Ift::Format2(map) = table else {
+            panic!("Not format 2.");
+        };
+
+        assert_eq!(map.cff_charstrings_offset(), Some(456));
+        assert_eq!(map.cff2_charstrings_offset(), None);
+
+        // Two offsets
+        let data = test_data::format2_with_two_charstrings_offset();
+        let table = Ift::read(FontData::new(&data)).unwrap();
+        let Ift::Format2(map) = table else {
+            panic!("Not format 2.");
+        };
+
+        assert_eq!(map.cff_charstrings_offset(), Some(456));
+        assert_eq!(map.cff2_charstrings_offset(), Some(789));
+    }
+
+    #[test]
+    fn invalid_format_number() {
+        // No offsets
+        let mut data = test_data::codepoints_only_format2();
+        data.write_at("format", 3u8);
+
+        let Err(err) = Ift::read(FontData::new(&data)) else {
+            panic!("Read should have failed due to invalid format number.");
+        };
+
+        assert_eq!(ReadError::InvalidFormat(3), err);
+    }
+
     #[test]
     fn compatibility_id() {
         let data = test_data::simple_format1();
@@ -434,17 +516,6 @@ mod tests {
         assert!(!map.is_entry_applied(2));
     }
 
-    #[test]
-    fn uri_template_as_string() {
-        let data = test_data::simple_format1();
-        let table = Ift::read(FontData::new(&data)).unwrap();
-        let Ift::Format1(map) = table else {
-            panic!("Not format 1.");
-        };
-
-        assert_eq!(Ok("ABCDEFɤ"), map.uri_template_as_string());
-    }
-
     #[test]
     fn glyph_keyed_glyph_data_for_one_table() {
         let data = test_data::glyf_u16_glyph_patches();
@@ -587,7 +658,7 @@ mod tests {
     #[test]
     fn glyph_keyed_glyph_data_for_one_table_gids_truncated() {
         let builder = test_data::glyf_u16_glyph_patches();
-        let len = builder.offset_for("table_count") as usize;
+        let len = builder.offset_for("table_count");
         let data = &builder.as_slice()[..len];
 
         let Err(err) = GlyphPatches::read(FontData::new(data), GlyphKeyedFlags::NONE) else {
@@ -599,7 +670,7 @@ mod tests {
     #[test]
     fn glyph_keyed_glyph_data_for_one_table_data_truncated() {
         let builder = test_data::glyf_u16_glyph_patches();
-        let len = builder.offset_for("gid_8_and_9_data") as usize;
+        let len = builder.offset_for("gid_8_and_9_data");
         let data = &builder.as_slice()[..len];
 
         let table = GlyphPatches::read(FontData::new(data), GlyphKeyedFlags::NONE).unwrap();
@@ -620,7 +691,7 @@ mod tests {
     #[test]
     fn glyph_keyed_glyph_data_for_one_table_offset_array_truncated() {
         let builder = test_data::glyf_u16_glyph_patches();
-        let len = builder.offset_for("gid_9_offset") as usize;
+        let len = builder.offset_for("gid_9_offset");
         let data = &builder.as_slice()[..len];
 
         let Err(err) = GlyphPatches::read(FontData::new(data), GlyphKeyedFlags::NONE) else {
