@@ -1,5 +1,7 @@
 //! Matching of glyph patterns.
 
+use core::cmp::max;
+
 use ttf_parser::opentype_layout::*;
 use ttf_parser::{GlyphId, LazyArray16};
 
@@ -22,7 +24,7 @@ pub fn match_input(
     input_len: u16,
     match_func: &match_func_t,
     end_position: &mut usize,
-    match_positions: &mut smallvec::SmallVec<[usize; 4]>,
+    match_positions: &mut [usize; MAX_CONTEXT_LENGTH],
     p_total_component_count: Option<&mut u8>,
 ) -> bool {
     // This is perhaps the trickiest part of OpenType...  Remarks:
@@ -59,20 +61,17 @@ pub fn match_input(
         return false;
     }
 
-    if count > match_positions.len() {
-        match_positions.resize(count, 0);
-    }
-
-    let mut iter = skipping_iterator_t::new(ctx, ctx.buffer.idx, false);
-    iter.set_glyph_data(0);
+    let mut iter = skipping_iterator_t::new(ctx, ctx.buffer.idx, input_len, false);
     iter.enable_matching(match_func);
 
     let first = ctx.buffer.cur(0);
     let first_lig_id = _hb_glyph_info_get_lig_id(first);
     let first_lig_comp = _hb_glyph_info_get_lig_comp(first);
-    let mut total_component_count = 0;
+    let mut total_component_count = _hb_glyph_info_get_lig_num_comps(first);
     let mut ligbase = Ligbase::NotChecked;
 
+    match_positions[0] = ctx.buffer.idx;
+
     for position in &mut match_positions[1..count] {
         let mut unsafe_to = 0;
         if !iter.next(Some(&mut unsafe_to)) {
@@ -106,7 +105,7 @@ pub fn match_input(
                         j -= 1;
                     }
 
-                    ligbase = if found && iter.may_skip(&out[j]) == may_skip_t::SKIP_YES {
+                    ligbase = if found && iter.may_skip(&out[j]) == Some(true) {
                         Ligbase::MaySkip
                     } else {
                         Ligbase::MayNotSkip
@@ -132,12 +131,9 @@ pub fn match_input(
     *end_position = iter.index() + 1;
 
     if let Some(p_total_component_count) = p_total_component_count {
-        total_component_count += _hb_glyph_info_get_lig_num_comps(first);
         *p_total_component_count = total_component_count;
     }
 
-    match_positions[0] = ctx.buffer.idx;
-
     true
 }
 
@@ -147,8 +143,7 @@ pub fn match_backtrack(
     match_func: &match_func_t,
     match_start: &mut usize,
 ) -> bool {
-    let mut iter = skipping_iterator_t::new(ctx, ctx.buffer.backtrack_len(), true);
-    iter.set_glyph_data(0);
+    let mut iter = skipping_iterator_t::new(ctx, ctx.buffer.backtrack_len(), backtrack_len, true);
     iter.enable_matching(match_func);
 
     for _ in 0..backtrack_len {
@@ -170,11 +165,7 @@ pub fn match_lookahead(
     start_index: usize,
     end_index: &mut usize,
 ) -> bool {
-    // Function should always be called with a non-zero starting index
-    // c.f. https://github.com/harfbuzz/rustybuzz/issues/142
-    assert!(start_index >= 1);
-    let mut iter = skipping_iterator_t::new(ctx, start_index - 1, true);
-    iter.set_glyph_data(0);
+    let mut iter = skipping_iterator_t::new(ctx, start_index - 1, lookahead_len, true);
     iter.enable_matching(match_func);
 
     for _ in 0..lookahead_len {
@@ -191,51 +182,24 @@ pub fn match_lookahead(
 
 pub type match_func_t<'a> = dyn Fn(GlyphId, u16) -> bool + 'a;
 
-// In harfbuzz, skipping iterator works quite differently than it works here. In harfbuzz,
-// hb_ot_apply_context contains a skipping iterator that itself contains another reference to
-// the apply_context, meaning that we have a circular reference. Due to ownership rules in Rust,
-// we cannot copy this approach. Because of this, we basically create a new skipping iterator
-// when needed, and we do not have the `reset` and `init` methods that exist in harfbuzz. This makes
-// backporting related changes very hard, but it seems unavoidable, unfortunately.
 pub struct skipping_iterator_t<'a, 'b> {
     ctx: &'a hb_ot_apply_context_t<'a, 'b>,
     lookup_props: u32,
     ignore_zwnj: bool,
     ignore_zwj: bool,
-    ignore_hidden: bool,
     mask: hb_mask_t,
     syllable: u8,
     matching: Option<&'a match_func_t<'a>>,
     buf_len: usize,
-    glyph_data: u16,
-    pub(crate) buf_idx: usize,
-}
-
-#[derive(PartialEq, Eq, Copy, Clone)]
-pub enum match_t {
-    MATCH,
-    NOT_MATCH,
-    SKIP,
-}
-
-#[derive(PartialEq, Eq, Copy, Clone)]
-enum may_match_t {
-    MATCH_NO,
-    MATCH_YES,
-    MATCH_MAYBE,
-}
-
-#[derive(PartialEq, Eq, Copy, Clone)]
-enum may_skip_t {
-    SKIP_NO,
-    SKIP_YES,
-    SKIP_MAYBE,
+    buf_idx: usize,
+    num_items: u16,
 }
 
 impl<'a, 'b> skipping_iterator_t<'a, 'b> {
     pub fn new(
         ctx: &'a hb_ot_apply_context_t<'a, 'b>,
         start_buf_index: usize,
+        num_items: u16,
         context_match: bool,
     ) -> Self {
         skipping_iterator_t {
@@ -245,33 +209,23 @@ impl<'a, 'b> skipping_iterator_t<'a, 'b> {
             ignore_zwnj: ctx.table_index == TableIndex::GPOS || (context_match && ctx.auto_zwnj),
             // Ignore ZWJ if we are matching context, or asked to.
             ignore_zwj: context_match || ctx.auto_zwj,
-            // Ignore hidden glyphs (like CGJ) during GPOS.
-            ignore_hidden: ctx.table_index == TableIndex::GPOS,
             mask: if context_match {
                 u32::MAX
             } else {
-                ctx.lookup_mask()
+                ctx.lookup_mask
             },
             syllable: if ctx.buffer.idx == start_buf_index && ctx.per_syllable {
                 ctx.buffer.cur(0).syllable()
             } else {
                 0
             },
-            glyph_data: 0,
             matching: None,
             buf_len: ctx.buffer.len,
             buf_idx: start_buf_index,
+            num_items,
         }
     }
 
-    pub fn set_glyph_data(&mut self, glyph_data: u16) {
-        self.glyph_data = glyph_data
-    }
-
-    fn advance_glyph_data(&mut self) {
-        self.glyph_data += 1;
-    }
-
     pub fn set_lookup_props(&mut self, lookup_props: u32) {
         self.lookup_props = lookup_props;
     }
@@ -285,25 +239,28 @@ impl<'a, 'b> skipping_iterator_t<'a, 'b> {
     }
 
     pub fn next(&mut self, unsafe_to: Option<&mut usize>) -> bool {
-        let stop = self.buf_len as i32 - 1;
-
-        while (self.buf_idx as i32) < stop {
+        assert!(self.num_items > 0);
+        while self.buf_idx + usize::from(self.num_items) < self.buf_len {
             self.buf_idx += 1;
             let info = &self.ctx.buffer.info[self.buf_idx];
 
-            match self.match_(info) {
-                match_t::MATCH => {
-                    self.advance_glyph_data();
-                    return true;
-                }
-                match_t::NOT_MATCH => {
-                    if let Some(unsafe_to) = unsafe_to {
-                        *unsafe_to = self.buf_idx + 1;
-                    }
+            let skip = self.may_skip(info);
+            if skip == Some(true) {
+                continue;
+            }
 
-                    return false;
+            let matched = self.may_match(info);
+            if matched == Some(true) || (matched.is_none() && skip == Some(false)) {
+                self.num_items -= 1;
+                return true;
+            }
+
+            if skip == Some(false) {
+                if let Some(unsafe_to) = unsafe_to {
+                    *unsafe_to = self.buf_idx + 1;
                 }
-                match_t::SKIP => continue,
+
+                return false;
             }
         }
 
@@ -315,27 +272,28 @@ impl<'a, 'b> skipping_iterator_t<'a, 'b> {
     }
 
     pub fn prev(&mut self, unsafe_from: Option<&mut usize>) -> bool {
-        let stop: usize = 0;
-
-        while self.buf_idx > stop {
+        assert!(self.num_items > 0);
+        while self.buf_idx >= usize::from(self.num_items) {
             self.buf_idx -= 1;
             let info = &self.ctx.buffer.out_info()[self.buf_idx];
 
-            match self.match_(info) {
-                match_t::MATCH => {
-                    self.advance_glyph_data();
-                    return true;
-                }
-                match_t::NOT_MATCH => {
-                    if let Some(unsafe_from) = unsafe_from {
-                        *unsafe_from = self.buf_idx.max(1) - 1;
-                    }
+            let skip = self.may_skip(info);
+            if skip == Some(true) {
+                continue;
+            }
 
-                    return false;
-                }
-                match_t::SKIP => {
-                    continue;
+            let matched = self.may_match(info);
+            if matched == Some(true) || (matched.is_none() && skip == Some(false)) {
+                self.num_items -= 1;
+                return true;
+            }
+
+            if skip == Some(false) {
+                if let Some(unsafe_from) = unsafe_from {
+                    *unsafe_from = max(1, self.buf_idx) - 1;
                 }
+
+                return false;
             }
         }
 
@@ -346,59 +304,33 @@ impl<'a, 'b> skipping_iterator_t<'a, 'b> {
         false
     }
 
-    pub fn match_(&self, info: &hb_glyph_info_t) -> match_t {
-        let skip = self.may_skip(info);
-
-        if skip == may_skip_t::SKIP_YES {
-            return match_t::SKIP;
-        }
-
-        let _match = self.may_match(info);
-
-        if _match == may_match_t::MATCH_YES
-            || (_match == may_match_t::MATCH_MAYBE && skip == may_skip_t::SKIP_NO)
-        {
-            return match_t::MATCH;
-        }
-
-        if skip == may_skip_t::SKIP_NO {
-            return match_t::NOT_MATCH;
-        }
-
-        match_t::SKIP
+    pub fn reject(&mut self) {
+        self.num_items += 1;
     }
 
-    fn may_match(&self, info: &hb_glyph_info_t) -> may_match_t {
-        if (info.mask & self.mask) == 0 || (self.syllable != 0 && self.syllable != info.syllable())
+    fn may_match(&self, info: &hb_glyph_info_t) -> Option<bool> {
+        if (info.mask & self.mask) != 0 && (self.syllable == 0 || self.syllable == info.syllable())
         {
-            return may_match_t::MATCH_NO;
+            self.matching.map(|f| f(info.as_glyph(), self.num_items))
+        } else {
+            Some(false)
         }
-
-        if let Some(match_func) = self.matching {
-            return if match_func(info.as_glyph(), self.glyph_data) {
-                may_match_t::MATCH_YES
-            } else {
-                may_match_t::MATCH_NO
-            };
-        }
-
-        may_match_t::MATCH_MAYBE
     }
 
-    fn may_skip(&self, info: &hb_glyph_info_t) -> may_skip_t {
+    fn may_skip(&self, info: &hb_glyph_info_t) -> Option<bool> {
         if !self.ctx.check_glyph_property(info, self.lookup_props) {
-            return may_skip_t::SKIP_YES;
+            return Some(true);
         }
 
-        if _hb_glyph_info_is_default_ignorable(info)
-            && (self.ignore_zwnj || !_hb_glyph_info_is_zwnj(info))
-            && (self.ignore_zwj || !_hb_glyph_info_is_zwj(info))
-            && (self.ignore_hidden || !_hb_glyph_info_is_hidden(info))
+        if !_hb_glyph_info_is_default_ignorable(info)
+            || info.is_hidden()
+            || (!self.ignore_zwnj && _hb_glyph_info_is_zwnj(info))
+            || (!self.ignore_zwj && _hb_glyph_info_is_zwj(info))
         {
-            return may_skip_t::SKIP_MAYBE;
+            return Some(false);
         }
 
-        may_skip_t::SKIP_NO
+        None
     }
 }
 
@@ -453,13 +385,14 @@ impl Apply for ContextLookup<'_> {
                 coverage.get(glyph)?;
                 let coverages_len = coverages.len();
 
-                let match_func = |glyph, index| {
+                let match_func = |glyph, num_items| {
+                    let index = coverages_len - num_items;
                     let coverage = coverages.get(index).unwrap();
                     coverage.get(glyph).is_some()
                 };
 
                 let mut match_end = 0;
-                let mut match_positions = smallvec::SmallVec::from_elem(0, 4);
+                let mut match_positions = [0; MAX_CONTEXT_LENGTH];
 
                 if match_input(
                     ctx,
@@ -478,11 +411,11 @@ impl Apply for ContextLookup<'_> {
                         match_end,
                         lookups,
                     );
-                    Some(())
+                    return Some(());
                 } else {
                     ctx.buffer
                         .unsafe_to_concat(Some(ctx.buffer.idx), Some(match_end));
-                    None
+                    return None;
                 }
             }
         }
@@ -529,8 +462,6 @@ impl SequenceRuleExt for SequenceRule<'_> {
 
     fn apply(&self, ctx: &mut hb_ot_apply_context_t, match_func: &match_func_t) -> Option<()> {
         apply_context(ctx, self.input, match_func, self.lookups)
-
-        // TODO: Port optimized version from https://github.com/harfbuzz/harfbuzz/commit/645fabd10
     }
 }
 
@@ -559,7 +490,7 @@ impl WouldApply for ChainedContextLookup<'_> {
                 ..
             } => {
                 (!ctx.zero_context
-                    || (backtrack_coverages.is_empty() && lookahead_coverages.is_empty()))
+                    || (backtrack_coverages.len() == 0 && lookahead_coverages.len() == 0))
                     && (ctx.glyphs.len() == usize::from(input_coverages.len()) + 1
                         && input_coverages
                             .into_iter()
@@ -607,24 +538,27 @@ impl Apply for ChainedContextLookup<'_> {
             } => {
                 coverage.get(glyph)?;
 
-                let back = |glyph, index| {
+                let back = |glyph, num_items| {
+                    let index = backtrack_coverages.len() - num_items;
                     let coverage = backtrack_coverages.get(index).unwrap();
                     coverage.contains(glyph)
                 };
 
-                let ahead = |glyph, index| {
+                let ahead = |glyph, num_items| {
+                    let index = lookahead_coverages.len() - num_items;
                     let coverage = lookahead_coverages.get(index).unwrap();
                     coverage.contains(glyph)
                 };
 
-                let input = |glyph, index| {
+                let input = |glyph, num_items| {
+                    let index = input_coverages.len() - num_items;
                     let coverage = input_coverages.get(index).unwrap();
                     coverage.contains(glyph)
                 };
 
                 let mut end_index = ctx.buffer.idx;
                 let mut match_end = 0;
-                let mut match_positions = smallvec::SmallVec::from_elem(0, 4);
+                let mut match_positions = [0; MAX_CONTEXT_LENGTH];
 
                 let input_matches = match_input(
                     ctx,
@@ -702,8 +636,6 @@ impl ChainRuleSetExt for ChainedSequenceRuleSet<'_> {
         } else {
             None
         }
-
-        // TODO: Port optimized version from https://github.com/harfbuzz/harfbuzz/commit/77080f86f
     }
 }
 
@@ -715,7 +647,7 @@ trait ChainRuleExt {
 
 impl ChainRuleExt for ChainedSequenceRule<'_> {
     fn would_apply(&self, ctx: &WouldApplyContext, match_func: &match_func_t) -> bool {
-        (!ctx.zero_context || (self.backtrack.is_empty() && self.lookahead.is_empty()))
+        (!ctx.zero_context || (self.backtrack.len() == 0 && self.lookahead.len() == 0))
             && (ctx.glyphs.len() == usize::from(self.input.len()) + 1
                 && self
                     .input
@@ -746,13 +678,14 @@ fn apply_context(
     match_func: &match_func_t,
     lookups: LazyArray16<SequenceLookupRecord>,
 ) -> Option<()> {
-    let match_func = |glyph, index| {
+    let match_func = |glyph, num_items| {
+        let index = input.len() - num_items;
         let value = input.get(index).unwrap();
         match_func(glyph, value)
     };
 
     let mut match_end = 0;
-    let mut match_positions = smallvec::SmallVec::from_elem(0, 4);
+    let mut match_positions = [0; MAX_CONTEXT_LENGTH];
 
     if match_input(
         ctx,
@@ -787,24 +720,27 @@ fn apply_chain_context(
 ) -> Option<()> {
     // NOTE: Whenever something in this method changes, we also need to
     // change it in the `apply` implementation for ChainedContextLookup.
-    let f1 = |glyph, index| {
+    let f1 = |glyph, num_items| {
+        let index = backtrack.len() - num_items;
         let value = backtrack.get(index).unwrap();
         match_funcs[0](glyph, value)
     };
 
-    let f2 = |glyph, index| {
+    let f2 = |glyph, num_items| {
+        let index = lookahead.len() - num_items;
         let value = lookahead.get(index).unwrap();
         match_funcs[2](glyph, value)
     };
 
-    let f3 = |glyph, index| {
+    let f3 = |glyph, num_items| {
+        let index = input.len() - num_items;
         let value = input.get(index).unwrap();
         match_funcs[1](glyph, value)
     };
 
     let mut end_index = ctx.buffer.idx;
     let mut match_end = 0;
-    let mut match_positions = smallvec::SmallVec::from_elem(0, 4);
+    let mut match_positions = [0; MAX_CONTEXT_LENGTH];
 
     let input_matches = match_input(
         ctx,
@@ -849,19 +785,15 @@ fn apply_chain_context(
 fn apply_lookup(
     ctx: &mut hb_ot_apply_context_t,
     input_len: usize,
-    match_positions: &mut smallvec::SmallVec<[usize; 4]>,
+    match_positions: &mut [usize; MAX_CONTEXT_LENGTH],
     match_end: usize,
     lookups: LazyArray16<SequenceLookupRecord>,
 ) {
     let mut count = input_len + 1;
 
-    if count > match_positions.len() {
-        match_positions.resize(count, 0);
-    }
-
     // All positions are distance from beginning of *output* buffer.
     // Adjust.
-    let mut end: isize = {
+    let mut end = {
         let backtrack_len = ctx.buffer.backtrack_len();
         let delta = backtrack_len as isize - ctx.buffer.idx as isize;
 
@@ -870,7 +802,7 @@ fn apply_lookup(
             match_positions[j] = (match_positions[j] as isize + delta) as _;
         }
 
-        backtrack_len as isize + match_end as isize - ctx.buffer.idx as isize
+        backtrack_len + match_end - ctx.buffer.idx
     };
 
     for record in lookups {
@@ -931,8 +863,8 @@ fn apply_lookup(
         //
         // It should be possible to construct tests for both of these cases.
 
-        end += delta;
-        if end < match_positions[idx] as isize {
+        end = (end as isize + delta) as _;
+        if end < match_positions[idx] {
             // End might end up being smaller than match_positions[idx] if the recursed
             // lookup ended up removing many items.
             // Just never rewind end beyond start of current position, since that is
@@ -941,8 +873,8 @@ fn apply_lookup(
             // https://bugs.chromium.org/p/chromium/issues/detail?id=659496
             // https://github.com/harfbuzz/harfbuzz/issues/1611
             //
-            delta += match_positions[idx] as isize - end;
-            end = match_positions[idx] as isize;
+            delta += match_positions[idx] as isize - end as isize;
+            end = match_positions[idx];
         }
 
         // next now is the position after the recursed lookup.
@@ -952,11 +884,6 @@ fn apply_lookup(
             if delta as usize + count > MAX_CONTEXT_LENGTH {
                 break;
             }
-
-            if delta as usize + count > match_positions.len() {
-                let inner_max = (core::cmp::max(4, match_positions.len()) as f32 * 1.5) as usize;
-                match_positions.resize(core::cmp::max(delta as usize + count, inner_max), 0);
-            }
         } else {
             // NOTE: delta is non-positive.
             delta = delta.max(next as isize - count as isize);
@@ -980,11 +907,11 @@ fn apply_lookup(
         }
     }
 
-    ctx.buffer.move_to(end.try_into().unwrap());
+    ctx.buffer.move_to(end);
 }
 
 /// Value represents glyph class.
-fn match_class(class_def: ClassDefinition<'_>) -> impl Fn(GlyphId, u16) -> bool + '_ {
+fn match_class<'a>(class_def: ClassDefinition<'a>) -> impl Fn(GlyphId, u16) -> bool + 'a {
     move |glyph, value| class_def.get(glyph) == value
 }
 
@@ -1007,13 +934,12 @@ pub struct WouldApplyContext<'a> {
 
 pub mod OT {
     use super::*;
-    use crate::hb::set_digest::{hb_set_digest_ext, hb_set_digest_t};
 
     pub struct hb_ot_apply_context_t<'a, 'b> {
         pub table_index: TableIndex,
         pub face: &'a hb_font_t<'b>,
         pub buffer: &'a mut hb_buffer_t,
-        lookup_mask: hb_mask_t,
+        pub lookup_mask: hb_mask_t,
         pub per_syllable: bool,
         pub lookup_index: LookupIndex,
         pub lookup_props: u32,
@@ -1022,9 +948,6 @@ pub mod OT {
         pub auto_zwj: bool,
         pub random: bool,
         pub random_state: u32,
-        pub last_base: i32,
-        pub last_base_until: u32,
-        pub digest: hb_set_digest_t,
     }
 
     impl<'a, 'b> hb_ot_apply_context_t<'a, 'b> {
@@ -1033,7 +956,6 @@ pub mod OT {
             face: &'a hb_font_t<'b>,
             buffer: &'a mut hb_buffer_t,
         ) -> Self {
-            let buffer_digest = buffer.digest();
             Self {
                 table_index,
                 face,
@@ -1047,9 +969,6 @@ pub mod OT {
                 auto_zwj: true,
                 random: false,
                 random_state: 1,
-                last_base: -1,
-                last_base_until: 0,
-                digest: buffer_digest,
             }
         }
 
@@ -1059,25 +978,13 @@ pub mod OT {
             self.random_state
         }
 
-        pub fn set_lookup_mask(&mut self, mask: hb_mask_t) {
-            self.lookup_mask = mask;
-            self.last_base = -1;
-            self.last_base_until = 0;
-        }
-
-        pub fn lookup_mask(&self) -> hb_mask_t {
-            self.lookup_mask
-        }
-
         pub fn recurse(&mut self, sub_lookup_index: LookupIndex) -> Option<()> {
             if self.nesting_level_left == 0 {
-                self.buffer.shaping_failed = true;
                 return None;
             }
 
             self.buffer.max_ops -= 1;
             if self.buffer.max_ops < 0 {
-                self.buffer.shaping_failed = true;
                 return None;
             }
 
@@ -1130,9 +1037,6 @@ pub mod OT {
                 // match_props has the set index.
                 if lookup_flags & lookup_flags::USE_MARK_FILTERING_SET != 0 {
                     let set_index = (match_props >> 16) as u16;
-                    // TODO: harfbuzz uses a digest here to speed things up if HB_NO_GDEF_CACHE
-                    // is enabled. But a bit harder to implement for us since it's taken care of by
-                    // ttf-parser
                     if let Some(table) = self.face.tables().gdef {
                         return table.is_mark_glyph(info.as_glyph(), Some(set_index));
                     } else {
@@ -1159,8 +1063,6 @@ pub mod OT {
             ligature: bool,
             component: bool,
         ) {
-            self.digest.add(glyph_id);
-
             let cur = self.buffer.cur_mut(0);
             let mut props = cur.glyph_props();
 
@@ -1188,13 +1090,16 @@ pub mod OT {
 
             if has_glyph_classes {
                 props &= GlyphPropsFlags::PRESERVE.bits();
-                cur.set_glyph_props(props | self.face.glyph_props(glyph_id));
+                props =
+                    (props & !GlyphPropsFlags::CLASS_MASK.bits()) | self.face.glyph_props(glyph_id);
             } else if !class_guess.is_empty() {
                 props &= GlyphPropsFlags::PRESERVE.bits();
-                cur.set_glyph_props(props | class_guess.bits());
+                props = (props & !GlyphPropsFlags::CLASS_MASK.bits()) | class_guess.bits();
             } else {
-                cur.set_glyph_props(props);
+                props = props & !GlyphPropsFlags::CLASS_MASK.bits();
             }
+
+            cur.set_glyph_props(props);
         }
 
         pub fn replace_glyph(&mut self, glyph_id: GlyphId) {
@@ -1234,7 +1139,7 @@ pub fn ligate_input(
     // Including the first glyph
     count: usize,
     // Including the first glyph
-    match_positions: &smallvec::SmallVec<[usize; 4]>,
+    match_positions: &[usize; MAX_CONTEXT_LENGTH],
     match_end: usize,
     total_component_count: u8,
     lig_glyph: GlyphId,
@@ -1319,9 +1224,6 @@ pub fn ligate_input(
                 if this_comp == 0 {
                     this_comp = last_num_comps;
                 }
-                // Avoid the potential for a wrap-around bug when subtracting from an unsigned integer
-                // c.f. https://github.com/harfbuzz/rustybuzz/issues/142
-                assert!(comps_so_far >= last_num_comps);
                 let new_lig_comp = comps_so_far - last_num_comps + this_comp.min(last_num_comps);
                 _hb_glyph_info_set_lig_props_for_mark(cur, lig_id, new_lig_comp);
             }
@@ -1350,9 +1252,6 @@ pub fn ligate_input(
                 break;
             }
 
-            // Avoid the potential for a wrap-around bug when subtracting from an unsigned integer
-            // c.f. https://github.com/harfbuzz/rustybuzz/issues/142
-            assert!(comps_so_far >= last_num_comps);
             let new_lig_comp = comps_so_far - last_num_comps + this_comp.min(last_num_comps);
             _hb_glyph_info_set_lig_props_for_mark(info, lig_id, new_lig_comp)
         }
