@@ -1,5 +1,7 @@
 use alloc::vec::Vec;
+use core::cmp::Ordering;
 use core::ops::Range;
+use ttf_parser::FromData;
 
 use ttf_parser::opentype_layout::{
     FeatureIndex, LanguageIndex, LookupIndex, ScriptIndex, VariationIndex,
@@ -19,7 +21,7 @@ pub struct hb_ot_map_t {
     stages: [Vec<StageMap>; 2],
 }
 
-#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 pub struct feature_map_t {
     tag: hb_tag_t,
     // GSUB/GPOS
@@ -35,6 +37,18 @@ pub struct feature_map_t {
     per_syllable: bool,
 }
 
+impl Ord for feature_map_t {
+    fn cmp(&self, other: &Self) -> Ordering {
+        self.tag.cmp(&other.tag)
+    }
+}
+
+impl PartialOrd for feature_map_t {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        self.tag.partial_cmp(&other.tag)
+    }
+}
+
 #[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub struct lookup_map_t {
     pub index: LookupIndex,
@@ -53,7 +67,9 @@ pub struct StageMap {
     pub pause_func: Option<pause_func_t>,
 }
 
-pub type pause_func_t = fn(&hb_ot_shape_plan_t, &hb_font_t, &mut hb_buffer_t);
+// Pause functions return true if new glyph indices might have been added to the buffer.
+// This is used to update buffer digest.
+pub type pause_func_t = fn(&hb_ot_shape_plan_t, &hb_font_t, &mut hb_buffer_t) -> bool;
 
 impl hb_ot_map_t {
     pub const MAX_BITS: u32 = 8;
@@ -165,6 +181,7 @@ pub struct hb_ot_map_builder_t<'a> {
     current_stage: [usize; 2],
     feature_infos: Vec<feature_info_t>,
     stages: [Vec<stage_info_t>; 2],
+    pub(crate) is_simple: bool,
 }
 
 #[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
@@ -186,6 +203,9 @@ struct stage_info_t {
     pause_func: Option<pause_func_t>,
 }
 
+const GLOBAL_BIT_SHIFT: u32 = 8 * u32::SIZE as u32 - 1;
+const GLOBAL_BIT_MASK: hb_mask_t = 1 << GLOBAL_BIT_SHIFT;
+
 impl<'a> hb_ot_map_builder_t<'a> {
     pub fn new(
         face: &'a hb_font_t<'a>,
@@ -222,6 +242,7 @@ impl<'a> hb_ot_map_builder_t<'a> {
             current_stage: [0, 0],
             feature_infos: Vec::new(),
             stages: [Vec::new(), Vec::new()],
+            is_simple: false,
         }
     }
 
@@ -230,6 +251,22 @@ impl<'a> hb_ot_map_builder_t<'a> {
         self.chosen_script[table_index]
     }
 
+    #[inline]
+    pub fn has_feature(&self, tag: hb_tag_t) -> bool {
+        for (table_index, table) in self.face.layout_tables() {
+            if let Some(script_index) = self.script_index[table_index] {
+                if table
+                    .find_language_feature(script_index, self.lang_index[table_index], tag)
+                    .is_some()
+                {
+                    return true;
+                }
+            }
+        }
+
+        false
+    }
+
     #[inline]
     pub fn add_feature(&mut self, tag: hb_tag_t, flags: hb_ot_map_feature_flags_t, value: u32) {
         if !tag.is_null() {
@@ -274,9 +311,6 @@ impl<'a> hb_ot_map_builder_t<'a> {
         self.current_stage[table_index] += 1;
     }
 
-    const GLOBAL_BIT_MASK: hb_mask_t = glyph_flag::DEFINED + 1;
-    const GLOBAL_BIT_SHIFT: u32 = glyph_flag::DEFINED.count_ones();
-
     pub fn compile(&mut self) -> hb_ot_map_t {
         // We default to applying required feature in stage 0.  If the required
         // feature has a tag that is known to the shaper, we apply required feature
@@ -318,8 +352,8 @@ impl<'a> hb_ot_map_builder_t<'a> {
     ) -> (Vec<feature_map_t>, [usize; 2], hb_mask_t) {
         let mut map_features = Vec::new();
         let mut required_stage = [0; 2];
-        let mut global_mask = Self::GLOBAL_BIT_MASK;
-        let mut next_bit = Self::GLOBAL_BIT_SHIFT + 1;
+        let mut global_mask = GLOBAL_BIT_MASK;
+        let mut next_bit = glyph_flag::DEFINED.count_ones() + 1;
 
         // Sort features and merge duplicates.
         self.dedup_feature_infos();
@@ -335,8 +369,7 @@ impl<'a> hb_ot_map_builder_t<'a> {
                 hb_ot_map_t::MAX_BITS.min(num_bits)
             };
 
-            let bits_available = 8 * core::mem::size_of::<hb_mask_t>() as u32;
-            if info.max_value == 0 || next_bit + bits_needed > bits_available {
+            if info.max_value == 0 || next_bit + bits_needed >= GLOBAL_BIT_SHIFT {
                 // Feature disabled, or not enough bits.
                 continue;
             }
@@ -374,7 +407,7 @@ impl<'a> hb_ot_map_builder_t<'a> {
 
             let (shift, mask) = if info.flags & F_GLOBAL != 0 && info.max_value == 1 {
                 // Uses the global bit
-                (Self::GLOBAL_BIT_SHIFT, Self::GLOBAL_BIT_MASK)
+                (GLOBAL_BIT_SHIFT, GLOBAL_BIT_MASK)
             } else {
                 let shift = next_bit;
                 let mask = (1 << (next_bit + bits_needed)) - (1 << next_bit);
@@ -397,6 +430,10 @@ impl<'a> hb_ot_map_builder_t<'a> {
             });
         }
 
+        if self.is_simple {
+            map_features.sort();
+        }
+
         (map_features, required_stage, global_mask)
     }
 
@@ -406,7 +443,9 @@ impl<'a> hb_ot_map_builder_t<'a> {
             return;
         }
 
-        feature_infos.sort();
+        if !self.is_simple {
+            feature_infos.sort();
+        }
 
         let mut j = 0;
         for i in 1..feature_infos.len() {
@@ -466,7 +505,7 @@ impl<'a> hb_ot_map_builder_t<'a> {
                             table_index,
                             feature_index,
                             variation_index,
-                            Self::GLOBAL_BIT_MASK,
+                            GLOBAL_BIT_MASK,
                             true,
                             true,
                             false,
@@ -497,7 +536,7 @@ impl<'a> hb_ot_map_builder_t<'a> {
                 let lookups = &mut map_lookups[table_index];
                 let len = lookups.len();
 
-                if last_lookup < len {
+                if last_lookup + 1 < len {
                     lookups[last_lookup..].sort();
 
                     let mut j = last_lookup;
