@@ -12,41 +12,36 @@ use super::{
     Brush, ColorPainter, ColorStop, PaintCachedColorGlyph, PaintError, Transform,
 };
 
-use alloc::vec::Vec;
+use crate::decycler::{Decycler, DecyclerError};
 
 #[cfg(feature = "libm")]
 #[allow(unused_imports)]
 use core_maths::*;
 
-#[cfg(any(test, feature = "std"))]
-mod visited_set {
-    pub type VisitedSet = std::collections::HashSet<usize>;
-}
+pub(crate) type PaintDecycler = Decycler<usize, MAX_TRAVERSAL_DEPTH>;
 
-#[cfg(not(any(test, feature = "std")))]
-mod visited_set {
-    /// A subset of the HashSet type that pretends every insertion is
-    /// new.
-    ///
-    /// This is used in `no_std` builds to represent a visited set that never
-    /// detects cycles. We rely only on a traversal depth check to avoid
-    /// infinite recursion instead.
-    #[derive(Default)]
-    pub struct VisitedSet {}
+// Avoid heap allocations for any gradient with <= 32 color stops. This number
+// was chosen to keep stack size < 512 bytes.
+//
+// The largest gradient in Noto Color Emoji has 13 stops.
+//
+// Only one ColorStopVec will be created per paint graph traversal.
+//
+// Usage of SmallVec as a response to Behdad's wonderful memory usage analysis:
+// <https://docs.google.com/document/d/1S47f3E--yqvFdG7lmmufxRoFi_wMzotC03v8UvS_p54/edit?tab=t.0#heading=h.bfj7urloz3oe>
+const MAX_INLINE_COLOR_STOPS: usize = 32;
 
-    impl VisitedSet {
-        /// Like HashSet, returns true if the value doesn't already exist in
-        /// the set. In our case, that's always.
-        pub fn insert(&mut self, _value: usize) -> bool {
-            true
+pub(crate) type ColorStopVec = crate::collections::SmallVec<ColorStop, MAX_INLINE_COLOR_STOPS>;
+
+impl From<DecyclerError> for PaintError {
+    fn from(value: DecyclerError) -> Self {
+        match value {
+            DecyclerError::CycleDetected => Self::PaintCycleDetected,
+            DecyclerError::DepthLimitExceeded => Self::DepthLimitExceeded,
         }
-
-        pub fn remove(&mut self, _value: &usize) {}
     }
 }
 
-pub use visited_set::VisitedSet;
-
 /// Depth at which we will stop traversing and return an error.
 ///
 /// Used to prevent stack overflows. Also allows us to avoid using a HashSet
@@ -55,7 +50,7 @@ pub use visited_set::VisitedSet;
 /// This limit matches the one used in HarfBuzz:
 /// HB_MAX_NESTING_LEVEL: <https://github.com/harfbuzz/harfbuzz/blob/c2f8f35a6cfce43b88552b3eb5c05062ac7007b2/src/hb-limits.hh#L53>
 /// hb_paint_context_t: <https://github.com/harfbuzz/harfbuzz/blob/c2f8f35a6cfce43b88552b3eb5c05062ac7007b2/src/OT/Color/COLR/COLR.hh#L74>
-const MAX_TRAVERSAL_DEPTH: u32 = 64;
+const MAX_TRAVERSAL_DEPTH: usize = 64;
 
 pub(crate) fn get_clipbox_font_units(
     colr_instance: &ColrInstance,
@@ -75,11 +70,17 @@ impl From<ResolvedColorStop> for ColorStop {
     }
 }
 
-fn make_sorted_resolved_stops(stops: &ColorStops, instance: &ColrInstance) -> Vec<ColorStop> {
+fn make_sorted_resolved_stops(
+    stops: &ColorStops,
+    instance: &ColrInstance,
+    out_stops: &mut ColorStopVec,
+) {
     let color_stop_iter = stops.resolve(instance).map(|stop| stop.into());
-    let mut collected: Vec<ColorStop> = color_stop_iter.collect();
-    collected.sort_by(|a, b| a.offset.partial_cmp(&b.offset).unwrap_or(Ordering::Equal));
-    collected
+    out_stops.clear();
+    for stop in color_stop_iter {
+        out_stops.push(stop);
+    }
+    out_stops.sort_by(|a, b| a.offset.partial_cmp(&b.offset).unwrap_or(Ordering::Equal));
 }
 
 struct CollectFillGlyphPainter<'a> {
@@ -153,8 +154,9 @@ pub(crate) fn traverse_with_callbacks(
     paint: &ResolvedPaint,
     instance: &ColrInstance,
     painter: &mut impl ColorPainter,
-    visited_set: &mut VisitedSet,
-    recurse_depth: u32,
+    decycler: &mut PaintDecycler,
+    resolved_stops: &mut ColorStopVec,
+    recurse_depth: usize,
 ) -> Result<(), PaintError> {
     if recurse_depth >= MAX_TRAVERSAL_DEPTH {
         return Err(PaintError::DepthLimitExceeded);
@@ -164,17 +166,15 @@ pub(crate) fn traverse_with_callbacks(
             for layer_index in range.clone() {
                 // Perform cycle detection with paint id here, second part of the tuple.
                 let (layer_paint, paint_id) = (*instance).v1_layer(layer_index)?;
-                if !visited_set.insert(paint_id) {
-                    return Err(PaintError::PaintCycleDetected);
-                }
+                let mut cycle_guard = decycler.enter(paint_id)?;
                 traverse_with_callbacks(
                     &resolve_paint(instance, &layer_paint)?,
                     instance,
                     painter,
-                    visited_set,
+                    &mut cycle_guard,
+                    resolved_stops,
                     recurse_depth + 1,
                 )?;
-                visited_set.remove(&paint_id);
             }
             Ok(())
         }
@@ -214,7 +214,7 @@ pub(crate) fn traverse_with_callbacks(
                 point_normalized
             };
 
-            let mut resolved_stops = make_sorted_resolved_stops(color_stops, instance);
+            make_sorted_resolved_stops(color_stops, instance, resolved_stops);
 
             // If p0p1 or p0p2 are degenerate probably nothing should be drawn.
             // If p0p1 and p0p2 are parallel then one side is the first color and the other side is
@@ -260,7 +260,7 @@ pub(crate) fn traverse_with_callbacks(
                     // the p0-p3 axis and result in specifying non-normalized color stops to the shader.
 
                     if color_stop_range == 0.0 && extend == &Extend::Pad {
-                        let mut extra_stop = last_stop.clone();
+                        let mut extra_stop = last_stop;
                         extra_stop.offset += 1.0;
                         resolved_stops.push(extra_stop);
 
@@ -280,7 +280,7 @@ pub(crate) fn traverse_with_callbacks(
                         let scale_factor = 1.0 / color_stop_range;
                         let start_offset = first_stop.offset;
 
-                        for stop in &mut resolved_stops {
+                        for stop in resolved_stops.iter_mut() {
                             stop.offset = (stop.offset - start_offset) * scale_factor;
                         }
                     }
@@ -311,7 +311,7 @@ pub(crate) fn traverse_with_callbacks(
             let mut radius0 = *radius0;
             let mut radius1 = *radius1;
 
-            let mut resolved_stops = make_sorted_resolved_stops(color_stops, instance);
+            make_sorted_resolved_stops(color_stops, instance, resolved_stops);
 
             match (
                 resolved_stops.first().cloned(),
@@ -329,7 +329,7 @@ pub(crate) fn traverse_with_callbacks(
                     // insert a color stop at the end. See LinearGradient for more details.
 
                     if color_stop_range == 0.0 && extend == &Extend::Pad {
-                        let mut extra_stop = last_stop.clone();
+                        let mut extra_stop = last_stop;
                         extra_stop.offset += 1.0;
                         resolved_stops.push(extra_stop);
                         color_stop_range = 1.0;
@@ -356,7 +356,7 @@ pub(crate) fn traverse_with_callbacks(
                         radius1 = radius0 + radius_diff * last_stop.offset;
                         radius0 += radius_diff * first_stop.offset;
 
-                        for stop in &mut resolved_stops {
+                        for stop in resolved_stops.iter_mut() {
                             stop.offset = (stop.offset - stops_start_offset) * scale_factor;
                         }
                     }
@@ -392,7 +392,7 @@ pub(crate) fn traverse_with_callbacks(
 
             let sector_angle = end_angle - start_angle;
 
-            let mut resolved_stops = make_sorted_resolved_stops(color_stops, instance);
+            make_sorted_resolved_stops(color_stops, instance, resolved_stops);
             if resolved_stops.is_empty() {
                 return Ok(());
             }
@@ -422,7 +422,7 @@ pub(crate) fn traverse_with_callbacks(
                     // the last color. Not adding this stop will skip the projection and result in
                     // specifying non-normalized color stops to the shader.
                     if color_stop_range == 0.0 && extend == &Extend::Pad {
-                        let mut offset_last = last_stop.clone();
+                        let mut offset_last = last_stop;
                         offset_last.offset += 1.0;
                         resolved_stops.push(offset_last);
                         color_stop_range = 1.0;
@@ -432,7 +432,7 @@ pub(crate) fn traverse_with_callbacks(
 
                     let scale_factor = 1.0 / color_stop_range;
 
-                    for shift_stop in &mut resolved_stops {
+                    for shift_stop in resolved_stops.iter_mut() {
                         shift_stop.offset = (shift_stop.offset - start_offset) * scale_factor;
                     }
 
@@ -451,7 +451,7 @@ pub(crate) fn traverse_with_callbacks(
                         (start_angle_scaled, end_angle_scaled) =
                             (end_angle_scaled, start_angle_scaled);
                         resolved_stops.reverse();
-                        for stop in &mut resolved_stops {
+                        for stop in resolved_stops.iter_mut() {
                             stop.offset = 1.0 - stop.offset;
                         }
                     }
@@ -482,7 +482,8 @@ pub(crate) fn traverse_with_callbacks(
                 &resolve_paint(instance, paint)?,
                 instance,
                 &mut optimizer,
-                visited_set,
+                decycler,
+                resolved_stops,
                 recurse_depth + 1,
             );
 
@@ -493,7 +494,8 @@ pub(crate) fn traverse_with_callbacks(
                     &resolve_paint(instance, paint)?,
                     instance,
                     painter,
-                    visited_set,
+                    decycler,
+                    resolved_stops,
                     recurse_depth + 1,
                 );
                 painter.pop_clip();
@@ -505,12 +507,9 @@ pub(crate) fn traverse_with_callbacks(
             let glyph_id = (*glyph_id).into();
             match (*instance).v1_base_glyph(glyph_id)? {
                 Some((base_glyph, base_glyph_paint_id)) => {
-                    if !visited_set.insert(base_glyph_paint_id) {
-                        return Err(PaintError::PaintCycleDetected);
-                    }
-
+                    let mut cycle_guard = decycler.enter(base_glyph_paint_id)?;
                     let draw_result = painter.paint_cached_color_glyph(glyph_id)?;
-                    let result = match draw_result {
+                    match draw_result {
                         PaintCachedColorGlyph::Ok => Ok(()),
                         PaintCachedColorGlyph::Unimplemented => {
                             let clipbox = get_clipbox_font_units(instance, glyph_id);
@@ -523,7 +522,8 @@ pub(crate) fn traverse_with_callbacks(
                                 &resolve_paint(instance, &base_glyph)?,
                                 instance,
                                 painter,
-                                visited_set,
+                                &mut cycle_guard,
+                                resolved_stops,
                                 recurse_depth + 1,
                             );
                             if clipbox.is_some() {
@@ -531,9 +531,7 @@ pub(crate) fn traverse_with_callbacks(
                             }
                             result
                         }
-                    };
-                    visited_set.remove(&base_glyph_paint_id);
-                    result
+                    }
                 }
                 None => Err(PaintError::GlyphNotFound(glyph_id)),
             }
@@ -558,7 +556,8 @@ pub(crate) fn traverse_with_callbacks(
                 &resolve_paint(instance, next_paint)?,
                 instance,
                 painter,
-                visited_set,
+                decycler,
+                resolved_stops,
                 recurse_depth + 1,
             );
             painter.pop_transform();
@@ -574,7 +573,8 @@ pub(crate) fn traverse_with_callbacks(
                 &resolve_paint(instance, backdrop_paint)?,
                 instance,
                 painter,
-                visited_set,
+                decycler,
+                resolved_stops,
                 recurse_depth + 1,
             );
             result?;
@@ -583,11 +583,12 @@ pub(crate) fn traverse_with_callbacks(
                 &resolve_paint(instance, source_paint)?,
                 instance,
                 painter,
-                visited_set,
+                decycler,
+                resolved_stops,
                 recurse_depth + 1,
             );
-            painter.pop_layer();
-            painter.pop_layer();
+            painter.pop_layer_with_mode(*mode);
+            painter.pop_layer_with_mode(CompositeMode::SrcOver);
             result
         }
     }
