@@ -395,7 +395,7 @@ impl<'a> OutlineGlyph<'a> {
         pen: &mut impl OutlinePen,
     ) -> Result<AdjustedMetrics, DrawError> {
         let ppem = size.ppem();
-        let coords = location.into().coords();
+        let coords = location.into().effective_coords();
         match &self.kind {
             OutlineKind::Glyf(glyf, outline) => {
                 with_glyf_memory(outline, Hinting::None, user_memory, |buf| {
@@ -446,7 +446,7 @@ impl<'a> OutlineGlyph<'a> {
         user_memory: Option<&mut [u8]>,
         sink: &mut impl unscaled::UnscaledOutlineSink,
     ) -> Result<i32, DrawError> {
-        let coords = location.into().coords();
+        let coords = location.into().effective_coords();
         let ppem = None;
         match &self.kind {
             OutlineKind::Glyf(glyf, outline) => {
@@ -641,6 +641,17 @@ impl<'a> OutlineGlyphCollection<'a> {
             .unwrap_or_default()
     }
 
+    /// Returns true when the font supports hinting at fractional sizes.
+    ///
+    /// When this returns false, the requested size will be rounded before
+    /// computing the scale factor for hinted glyphs.
+    pub fn fractional_size_hinting(&self) -> bool {
+        match &self.kind {
+            OutlineCollectionKind::Glyf(glyf) => glyf.fractional_size_hinting,
+            _ => true,
+        }
+    }
+
     pub(crate) fn font(&self) -> Option<&FontRef<'a>> {
         match &self.kind {
             OutlineCollectionKind::Glyf(glyf) => Some(&glyf.font),
@@ -687,7 +698,7 @@ pub(super) fn with_glyf_memory<R>(
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{instance::Location, MetadataProvider};
+    use crate::{instance::Location, outline::pen::SvgPen, MetadataProvider};
     use kurbo::{Affine, BezPath, PathEl, Point};
     use read_fonts::{types::GlyphId, FontRef, TableProvider};
 
@@ -1442,4 +1453,41 @@ mod tests {
             .unwrap();
         assert_eq!(advance, 11.0);
     }
+
+    /// Ensure we produce different TrueType outlines based on the
+    /// fractional_size_hinting flag
+    #[test]
+    fn fractional_size_hinting_matters() {
+        let font = FontRef::from_index(font_test_data::TINOS_SUBSET, 0).unwrap();
+        let mut outlines = font.outline_glyphs();
+        let instance = HintingInstance::new(
+            &outlines,
+            Size::new(24.8),
+            LocationRef::default(),
+            HintingOptions::default(),
+        )
+        .unwrap();
+        let gid = GlyphId::new(2);
+        let outline_with_fractional = {
+            let OutlineCollectionKind::Glyf(glyf) = &mut outlines.kind else {
+                panic!("this is definitely a TrueType font");
+            };
+            glyf.fractional_size_hinting = true;
+            let mut pen = SvgPen::new();
+            let outline = outlines.get(gid).unwrap();
+            outline.draw(&instance, &mut pen).unwrap();
+            pen.to_string()
+        };
+        let outline_without_fractional = {
+            let OutlineCollectionKind::Glyf(glyf) = &mut outlines.kind else {
+                panic!("this is definitely a TrueType font");
+            };
+            glyf.fractional_size_hinting = false;
+            let mut pen = SvgPen::new();
+            let outline = outlines.get(gid).unwrap();
+            outline.draw(&instance, &mut pen).unwrap();
+            pen.to_string()
+        };
+        assert_ne!(outline_with_fractional, outline_without_fractional);
+    }
 }
