@@ -1,14 +1,12 @@
 //! Definitions of name-related helpers and newtypes, primarily for the
 //! component model.
 
-use crate::prelude::*;
 use crate::{Result, WasmFeatures};
-use core::borrow::Borrow;
-use core::cmp::Ordering;
-use core::fmt;
-use core::hash::{Hash, Hasher};
-use core::ops::Deref;
 use semver::Version;
+use std::borrow::Borrow;
+use std::fmt;
+use std::hash::{Hash, Hasher};
+use std::ops::Deref;
 
 /// Represents a kebab string slice used in validation.
 ///
@@ -40,7 +38,7 @@ impl KebabStr {
         // Therefore transmuting `&str` to `&KebabStr` is safe.
         #[allow(unsafe_code)]
         unsafe {
-            core::mem::transmute::<_, &Self>(s.as_ref())
+            std::mem::transmute::<_, &Self>(s.as_ref())
         }
     }
 
@@ -102,20 +100,6 @@ impl PartialEq<KebabString> for KebabStr {
     }
 }
 
-impl Ord for KebabStr {
-    fn cmp(&self, other: &Self) -> Ordering {
-        let self_chars = self.chars().map(|c| c.to_ascii_lowercase());
-        let other_chars = other.chars().map(|c| c.to_ascii_lowercase());
-        self_chars.cmp(other_chars)
-    }
-}
-
-impl PartialOrd for KebabStr {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        Some(self.cmp(other))
-    }
-}
-
 impl Hash for KebabStr {
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.len().hash(state);
@@ -190,18 +174,6 @@ impl Borrow<KebabStr> for KebabString {
     }
 }
 
-impl Ord for KebabString {
-    fn cmp(&self, other: &Self) -> Ordering {
-        self.as_kebab_str().cmp(other.as_kebab_str())
-    }
-}
-
-impl PartialOrd for KebabString {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        self.as_kebab_str().partial_cmp(other.as_kebab_str())
-    }
-}
-
 impl PartialEq for KebabString {
     fn eq(&self, other: &Self) -> bool {
         self.as_kebab_str().eq(other.as_kebab_str())
@@ -241,9 +213,6 @@ impl From<KebabString> for String {
 /// * a plain method name : `[method]a-b.c-d`
 /// * a plain static method name : `[static]a-b.c-d`
 /// * a plain constructor: `[constructor]a-b`
-/// * an async plain label: `[async]a-b-c`
-/// * an async plain method name : `[async method]a-b.c-d`
-/// * an async plain static method name : `[async static]a-b.c-d`
 /// * an interface name: `wasi:cli/reactor@0.1.0`
 /// * a dependency name: `locked-dep=foo:bar/baz`
 /// * a URL name: `url=https://..`
@@ -260,7 +229,7 @@ pub struct ComponentName {
     kind: ParsedComponentNameKind,
 }
 
-#[derive(Copy, Clone, PartialEq, Eq, PartialOrd, Ord)]
+#[derive(Copy, Clone)]
 enum ParsedComponentNameKind {
     Label,
     Constructor,
@@ -270,9 +239,6 @@ enum ParsedComponentNameKind {
     Dependency,
     Url,
     Hash,
-    AsyncLabel,
-    AsyncMethod,
-    AsyncStatic,
 }
 
 /// Created via [`ComponentName::kind`] and classifies a name.
@@ -300,28 +266,24 @@ pub enum ComponentNameKind<'a> {
     /// `integrity=sha256:...`
     #[allow(missing_docs)]
     Hash(HashName<'a>),
-    /// `[async]a-b-c`
-    AsyncLabel(&'a KebabStr),
-    /// `[async method]a-b.c-d`
-    #[allow(missing_docs)]
-    AsyncMethod(ResourceFunc<'a>),
-    /// `[async static]a-b.c-d`
-    #[allow(missing_docs)]
-    AsyncStatic(ResourceFunc<'a>),
 }
 
 const CONSTRUCTOR: &str = "[constructor]";
 const METHOD: &str = "[method]";
 const STATIC: &str = "[static]";
-const ASYNC: &str = "[async]";
-const ASYNC_METHOD: &str = "[async method]";
-const ASYNC_STATIC: &str = "[async static]";
 
 impl ComponentName {
     /// Attempts to parse `name` as a valid component name, returning `Err` if
     /// it's not valid.
     pub fn new(name: &str, offset: usize) -> Result<ComponentName> {
-        Self::new_with_features(name, offset, WasmFeatures::default())
+        Self::new_with_features(
+            name,
+            offset,
+            WasmFeatures {
+                component_model: true,
+                ..Default::default()
+            },
+        )
     }
 
     /// Attempts to parse `name` as a valid component name, returning `Err` if
@@ -351,12 +313,9 @@ impl ComponentName {
         use ParsedComponentNameKind as PK;
         match self.kind {
             PK::Label => Label(KebabStr::new_unchecked(&self.raw)),
-            PK::AsyncLabel => AsyncLabel(KebabStr::new_unchecked(&self.raw[ASYNC.len()..])),
             PK::Constructor => Constructor(KebabStr::new_unchecked(&self.raw[CONSTRUCTOR.len()..])),
             PK::Method => Method(ResourceFunc(&self.raw[METHOD.len()..])),
-            PK::AsyncMethod => AsyncMethod(ResourceFunc(&self.raw[ASYNC_METHOD.len()..])),
             PK::Static => Static(ResourceFunc(&self.raw[STATIC.len()..])),
-            PK::AsyncStatic => AsyncStatic(ResourceFunc(&self.raw[ASYNC_STATIC.len()..])),
             PK::Interface => Interface(InterfaceName(&self.raw)),
             PK::Dependency => Dependency(DependencyName(&self.raw)),
             PK::Url => Url(UrlName(&self.raw)),
@@ -390,18 +349,6 @@ impl PartialEq for ComponentName {
 
 impl Eq for ComponentName {}
 
-impl Ord for ComponentName {
-    fn cmp(&self, other: &ComponentName) -> Ordering {
-        self.kind().cmp(&other.kind())
-    }
-}
-
-impl PartialOrd for ComponentName {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        self.kind.partial_cmp(&other.kind)
-    }
-}
-
 impl fmt::Display for ComponentName {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         self.raw.fmt(f)
@@ -414,72 +361,14 @@ impl fmt::Debug for ComponentName {
     }
 }
 
-impl ComponentNameKind<'_> {
-    /// Returns the [`ParsedComponentNameKind`] of the [`ComponentNameKind`].
-    fn kind(&self) -> ParsedComponentNameKind {
-        match self {
-            Self::Label(_) => ParsedComponentNameKind::Label,
-            Self::Constructor(_) => ParsedComponentNameKind::Constructor,
-            Self::Method(_) => ParsedComponentNameKind::Method,
-            Self::Static(_) => ParsedComponentNameKind::Static,
-            Self::Interface(_) => ParsedComponentNameKind::Interface,
-            Self::Dependency(_) => ParsedComponentNameKind::Dependency,
-            Self::Url(_) => ParsedComponentNameKind::Url,
-            Self::Hash(_) => ParsedComponentNameKind::Hash,
-            Self::AsyncLabel(_) => ParsedComponentNameKind::AsyncLabel,
-            Self::AsyncMethod(_) => ParsedComponentNameKind::AsyncMethod,
-            Self::AsyncStatic(_) => ParsedComponentNameKind::AsyncStatic,
-        }
-    }
-}
-
-impl Ord for ComponentNameKind<'_> {
-    fn cmp(&self, other: &Self) -> Ordering {
-        use ComponentNameKind::*;
-
-        match (self, other) {
-            (Label(lhs) | AsyncLabel(lhs), Label(rhs) | AsyncLabel(rhs)) => lhs.cmp(rhs),
-            (Constructor(lhs), Constructor(rhs)) => lhs.cmp(rhs),
-            (
-                Method(lhs) | AsyncMethod(lhs) | Static(lhs) | AsyncStatic(lhs),
-                Method(rhs) | AsyncMethod(rhs) | Static(rhs) | AsyncStatic(rhs),
-            ) => lhs.cmp(rhs),
-            (Interface(lhs), Interface(rhs)) => lhs.cmp(rhs),
-            (Dependency(lhs), Dependency(rhs)) => lhs.cmp(rhs),
-            (Url(lhs), Url(rhs)) => lhs.cmp(rhs),
-            (Hash(lhs), Hash(rhs)) => lhs.cmp(rhs),
-
-            (Label(_), _)
-            | (AsyncLabel(_), _)
-            | (Constructor(_), _)
-            | (Method(_), _)
-            | (Static(_), _)
-            | (AsyncMethod(_), _)
-            | (AsyncStatic(_), _)
-            | (Interface(_), _)
-            | (Dependency(_), _)
-            | (Url(_), _)
-            | (Hash(_), _) => self.kind().cmp(&other.kind()),
-        }
-    }
-}
-
-impl PartialOrd for ComponentNameKind<'_> {
-    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
-        Some(self.cmp(other))
-    }
-}
-
 impl Hash for ComponentNameKind<'_> {
     fn hash<H: Hasher>(&self, hasher: &mut H) {
         use ComponentNameKind::*;
         match self {
-            Label(name) | AsyncLabel(name) => (0u8, name).hash(hasher),
+            Label(name) => (0u8, name).hash(hasher),
             Constructor(name) => (1u8, name).hash(hasher),
             // for hashing method == static
-            Method(name) | Static(name) | AsyncMethod(name) | AsyncStatic(name) => {
-                (2u8, name).hash(hasher)
-            }
+            Method(name) | Static(name) => (2u8, name).hash(hasher),
             Interface(name) => (3u8, name).hash(hasher),
             Dependency(name) => (4u8, name).hash(hasher),
             Url(name) => (5u8, name).hash(hasher),
@@ -492,19 +381,20 @@ impl PartialEq for ComponentNameKind<'_> {
     fn eq(&self, other: &ComponentNameKind<'_>) -> bool {
         use ComponentNameKind::*;
         match (self, other) {
-            (Label(a) | AsyncLabel(a), Label(b) | AsyncLabel(b)) => a == b,
-            (Label(_) | AsyncLabel(_), _) => false,
+            (Label(a), Label(b)) => a == b,
+            (Label(_), _) => false,
             (Constructor(a), Constructor(b)) => a == b,
             (Constructor(_), _) => false,
 
             // method == static for the purposes of hashing so equate them here
             // as well.
-            (
-                Method(a) | Static(a) | AsyncMethod(a) | AsyncStatic(a),
-                Method(b) | Static(b) | AsyncMethod(b) | AsyncStatic(b),
-            ) => a == b,
+            (Method(a), Method(b))
+            | (Static(a), Static(b))
+            | (Method(a), Static(b))
+            | (Static(a), Method(b)) => a == b,
 
-            (Method(_) | Static(_) | AsyncMethod(_) | AsyncStatic(_), _) => false,
+            (Method(_), _) => false,
+            (Static(_), _) => false,
 
             (Interface(a), Interface(b)) => a == b,
             (Interface(_), _) => false,
@@ -521,7 +411,7 @@ impl PartialEq for ComponentNameKind<'_> {
 impl Eq for ComponentNameKind<'_> {}
 
 /// A resource name and its function, stored as `a.b`.
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub struct ResourceFunc<'a>(&'a str);
 
 impl<'a> ResourceFunc<'a> {
@@ -538,7 +428,7 @@ impl<'a> ResourceFunc<'a> {
 }
 
 /// An interface name, stored as `a:b/c@1.2.3`
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub struct InterfaceName<'a>(&'a str);
 
 impl<'a> InterfaceName<'a> {
@@ -583,7 +473,7 @@ impl<'a> InterfaceName<'a> {
 
 /// A dependency on an implementation either as `locked-dep=...` or
 /// `unlocked-dep=...`
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub struct DependencyName<'a>(&'a str);
 
 impl<'a> DependencyName<'a> {
@@ -593,8 +483,9 @@ impl<'a> DependencyName<'a> {
     }
 }
 
-/// A dependency on an implementation either as `url=...`
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+/// A dependency on an implementation either as `url=...` or
+/// `relative-url=...`
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub struct UrlName<'a>(&'a str);
 
 impl<'a> UrlName<'a> {
@@ -605,7 +496,7 @@ impl<'a> UrlName<'a> {
 }
 
 /// A dependency on an implementation either as `integrity=...`.
-#[derive(Debug, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
+#[derive(Debug, Clone, Hash, Eq, PartialEq)]
 pub struct HashName<'a>(&'a str);
 
 impl<'a> HashName<'a> {
@@ -628,10 +519,6 @@ struct ComponentNameParser<'a> {
 
 impl<'a> ComponentNameParser<'a> {
     fn parse(&mut self) -> Result<ParsedComponentNameKind> {
-        if self.eat_str(ASYNC) {
-            self.expect_kebab()?;
-            return Ok(ParsedComponentNameKind::AsyncLabel);
-        }
         if self.eat_str(CONSTRUCTOR) {
             self.expect_kebab()?;
             return Ok(ParsedComponentNameKind::Constructor);
@@ -648,18 +535,6 @@ impl<'a> ComponentNameParser<'a> {
             self.expect_kebab()?;
             return Ok(ParsedComponentNameKind::Static);
         }
-        if self.eat_str(ASYNC_METHOD) {
-            let resource = self.take_until('.')?;
-            self.kebab(resource)?;
-            self.expect_kebab()?;
-            return Ok(ParsedComponentNameKind::AsyncMethod);
-        }
-        if self.eat_str(ASYNC_STATIC) {
-            let resource = self.take_until('.')?;
-            self.kebab(resource)?;
-            self.expect_kebab()?;
-            return Ok(ParsedComponentNameKind::AsyncStatic);
-        }
 
         // 'unlocked-dep=<' <pkgnamequery> '>'
         if self.eat_str("unlocked-dep=") {
@@ -689,6 +564,17 @@ impl<'a> ComponentNameParser<'a> {
             self.eat_optional_hash()?;
             return Ok(ParsedComponentNameKind::Url);
         }
+        // 'relative-url=<' <nonbrackets> '>' (',' <hashname>)?
+        if self.eat_str("relative-url=") {
+            self.expect_str("<")?;
+            let url = self.take_up_to('>')?;
+            if url.contains('<') {
+                bail!(self.offset, "relative-url cannot contain `<`");
+            }
+            self.expect_str(">")?;
+            self.eat_optional_hash()?;
+            return Ok(ParsedComponentNameKind::Url);
+        }
 
         // 'integrity=<' <integrity-metadata> '>'
         if self.eat_str("integrity=") {
@@ -744,15 +630,15 @@ impl<'a> ComponentNameParser<'a> {
     // pkgpath ::= <namespace>+ <label> <projection>*
     fn pkg_path(&mut self, require_projection: bool) -> Result<()> {
         // There must be at least one package namespace
-        self.take_lowercase_kebab()?;
+        self.take_kebab()?;
         self.expect_str(":")?;
-        self.take_lowercase_kebab()?;
+        self.take_kebab()?;
 
-        if self.features.component_model_nested_names() {
+        if self.features.component_model_nested_names {
             // Take the remaining package namespaces and name
             while self.next.starts_with(':') {
                 self.expect_str(":")?;
-                self.take_lowercase_kebab()?;
+                self.take_kebab()?;
             }
         }
 
@@ -761,7 +647,7 @@ impl<'a> ComponentNameParser<'a> {
             self.expect_str("/")?;
             self.take_kebab()?;
 
-            if self.features.component_model_nested_names() {
+            if self.features.component_model_nested_names {
                 while self.next.starts_with('/') {
                     self.expect_str("/")?;
                     self.take_kebab()?;
@@ -934,20 +820,6 @@ impl<'a> ComponentNameParser<'a> {
             .unwrap_or_else(|| self.expect_kebab())
     }
 
-    fn take_lowercase_kebab(&mut self) -> Result<&'a KebabStr> {
-        let kebab = self.take_kebab()?;
-        if let Some(c) = kebab
-            .chars()
-            .find(|c| c.is_alphabetic() && !c.is_lowercase())
-        {
-            bail!(
-                self.offset,
-                "character `{c}` is not lowercase in package name/namespace"
-            );
-        }
-        Ok(kebab)
-    }
-
     fn expect_kebab(&mut self) -> Result<&'a KebabStr> {
         let s = self.take_rest();
         self.kebab(s)
