@@ -204,14 +204,13 @@ pub unsafe fn ORStart() -> super::super::super::Win32::Foundation::WIN32_ERROR {
 }
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct ORHKEY(pub *mut core::ffi::c_void);
+pub struct ORHKEY(pub isize);
 impl ORHKEY {
     pub fn is_invalid(&self) -> bool {
-        self.0 == -1 as _ || self.0 == 0 as _
+        self.0 == -1 || self.0 == 0
     }
 }
 impl windows_core::Free for ORHKEY {
-    #[inline]
     unsafe fn free(&mut self) {
         if !self.is_invalid() {
             _ = ORCloseKey(*self);
