@@ -432,10 +432,10 @@ impl Default for AMSI_UAC_REQUEST_PACKAGED_APP_INFO {
 pub const CAntimalware: windows_core::GUID = windows_core::GUID::from_u128(0xfdb00e52_a214_4aa1_8fba_4357bb0072ec);
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct HAMSICONTEXT(pub isize);
+pub struct HAMSICONTEXT(pub *mut core::ffi::c_void);
 impl HAMSICONTEXT {
     pub fn is_invalid(&self) -> bool {
-        self.0 == -1 || self.0 == 0
+        self.0 == -1 as _ || self.0 == 0 as _
     }
 }
 impl Default for HAMSICONTEXT {
@@ -448,10 +448,10 @@ impl windows_core::TypeKind for HAMSICONTEXT {
 }
 #[repr(transparent)]
 #[derive(Clone, Copy, Debug, PartialEq, Eq)]
-pub struct HAMSISESSION(pub isize);
+pub struct HAMSISESSION(pub *mut core::ffi::c_void);
 impl HAMSISESSION {
     pub fn is_invalid(&self) -> bool {
-        self.0 == -1 || self.0 == 0
+        self.0 == -1 as _ || self.0 == 0 as _
     }
 }
 impl Default for HAMSISESSION {
