@@ -1,4 +1,5 @@
 use super::*;
+use core::mem::transmute;
 
 /// A VARIANT ([VARIANT](https://learn.microsoft.com/en-us/windows/win32/api/oaidl/ns-oaidl-variant)) is a container that can store different types of values.
 #[repr(transparent)]
@@ -53,15 +54,15 @@ impl Drop for PROPVARIANT {
 }
 
 impl TypeKind for VARIANT {
-    type TypeKind = ValueType;
+    type TypeKind = CloneType;
 }
 
 impl TypeKind for PROPVARIANT {
-    type TypeKind = ValueType;
+    type TypeKind = CloneType;
 }
 
-impl std::fmt::Debug for VARIANT {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl core::fmt::Debug for VARIANT {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         let mut debug = f.debug_struct("VARIANT");
         debug.field("type", &unsafe { self.0.Anonymous.Anonymous.vt });
 
@@ -73,8 +74,8 @@ impl std::fmt::Debug for VARIANT {
     }
 }
 
-impl std::fmt::Debug for PROPVARIANT {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+impl core::fmt::Debug for PROPVARIANT {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
         let mut debug = f.debug_struct("PROPVARIANT");
         debug.field("type", &unsafe { self.0.Anonymous.Anonymous.vt });
 
@@ -86,15 +87,15 @@ impl std::fmt::Debug for PROPVARIANT {
     }
 }
 
-impl std::fmt::Display for VARIANT {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        std::write!(f, "{}", BSTR::try_from(self).unwrap_or_default())
+impl core::fmt::Display for VARIANT {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        core::write!(f, "{}", BSTR::try_from(self).unwrap_or_default())
     }
 }
 
-impl std::fmt::Display for PROPVARIANT {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        std::write!(f, "{}", BSTR::try_from(self).unwrap_or_default())
+impl core::fmt::Display for PROPVARIANT {
+    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
+        core::write!(f, "{}", BSTR::try_from(self).unwrap_or_default())
     }
 }
 
@@ -136,7 +137,7 @@ impl VARIANT {
     ///
     /// This function does not allocate memory.
     pub fn new() -> Self {
-        unsafe { std::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 
     /// Returns true if the `VARIANT` is empty.
@@ -162,9 +163,9 @@ impl VARIANT {
 impl PROPVARIANT {
     /// Create an empty `PROPVARIANT`.
     ///
-    /// This function does not allocate memory.    
+    /// This function does not allocate memory.
     pub fn new() -> Self {
-        unsafe { std::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     }
 
     /// Returns true if the `PROPVARIANT` is empty.
@@ -213,7 +214,15 @@ impl From<IUnknown> for VARIANT {
     fn from(value: IUnknown) -> Self {
         Self(imp::VARIANT {
             Anonymous: imp::VARIANT_0 {
-                Anonymous: imp::VARIANT_0_0 { vt: imp::VT_UNKNOWN, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { punkVal: value.into_raw() } },
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_UNKNOWN,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 {
+                        punkVal: value.into_raw(),
+                    },
+                },
             },
         })
     }
@@ -223,7 +232,15 @@ impl From<IUnknown> for PROPVARIANT {
     fn from(value: IUnknown) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_UNKNOWN, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { punkVal: value.into_raw() } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_UNKNOWN,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 {
+                        punkVal: value.into_raw(),
+                    },
+                },
             },
         })
     }
@@ -233,8 +250,10 @@ impl TryFrom<&VARIANT> for IUnknown {
     type Error = Error;
     fn try_from(from: &VARIANT) -> Result<Self> {
         unsafe {
-            if from.0.Anonymous.Anonymous.vt == imp::VT_UNKNOWN && !from.0.Anonymous.Anonymous.Anonymous.punkVal.is_null() {
-                let unknown: &IUnknown = std::mem::transmute(&from.0.Anonymous.Anonymous.Anonymous.punkVal);
+            if from.0.Anonymous.Anonymous.vt == imp::VT_UNKNOWN
+                && !from.0.Anonymous.Anonymous.Anonymous.punkVal.is_null()
+            {
+                let unknown: &IUnknown = transmute(&from.0.Anonymous.Anonymous.Anonymous.punkVal);
                 Ok(unknown.clone())
             } else {
                 Err(Error::from_hresult(imp::TYPE_E_TYPEMISMATCH))
@@ -247,8 +266,10 @@ impl TryFrom<&PROPVARIANT> for IUnknown {
     type Error = Error;
     fn try_from(from: &PROPVARIANT) -> Result<Self> {
         unsafe {
-            if from.0.Anonymous.Anonymous.vt == imp::VT_UNKNOWN && !from.0.Anonymous.Anonymous.Anonymous.punkVal.is_null() {
-                let unknown: &IUnknown = std::mem::transmute(&from.0.Anonymous.Anonymous.Anonymous.punkVal);
+            if from.0.Anonymous.Anonymous.vt == imp::VT_UNKNOWN
+                && !from.0.Anonymous.Anonymous.Anonymous.punkVal.is_null()
+            {
+                let unknown: &IUnknown = transmute(&from.0.Anonymous.Anonymous.Anonymous.punkVal);
                 Ok(unknown.clone())
             } else {
                 Err(Error::from_hresult(imp::TYPE_E_TYPEMISMATCH))
@@ -263,7 +284,15 @@ impl From<BSTR> for VARIANT {
     fn from(value: BSTR) -> Self {
         Self(imp::VARIANT {
             Anonymous: imp::VARIANT_0 {
-                Anonymous: imp::VARIANT_0_0 { vt: imp::VT_BSTR, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { bstrVal: value.into_raw() } },
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_BSTR,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 {
+                        bstrVal: value.into_raw(),
+                    },
+                },
             },
         })
     }
@@ -273,7 +302,15 @@ impl From<BSTR> for PROPVARIANT {
     fn from(value: BSTR) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_BSTR, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { bstrVal: value.into_raw() } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_BSTR,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 {
+                        bstrVal: value.into_raw(),
+                    },
+                },
             },
         })
     }
@@ -303,7 +340,8 @@ impl TryFrom<&PROPVARIANT> for BSTR {
     type Error = Error;
     fn try_from(from: &PROPVARIANT) -> Result<Self> {
         let mut value = Self::new();
-        HRESULT(unsafe { imp::PropVariantToBSTR(&from.0, &mut value as *mut _ as *mut _) }).map(|| value)
+        HRESULT(unsafe { imp::PropVariantToBSTR(&from.0, &mut value as *mut _ as *mut _) })
+            .map(|| value)
     }
 }
 
@@ -313,7 +351,15 @@ impl From<bool> for VARIANT {
     fn from(value: bool) -> Self {
         Self(imp::VARIANT {
             Anonymous: imp::VARIANT_0 {
-                Anonymous: imp::VARIANT_0_0 { vt: imp::VT_BOOL, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { boolVal: if value { -1 } else { 0 } } },
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_BOOL,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 {
+                        boolVal: if value { -1 } else { 0 },
+                    },
+                },
             },
         })
     }
@@ -323,7 +369,15 @@ impl From<bool> for PROPVARIANT {
     fn from(value: bool) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_BOOL, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { boolVal: if value { -1 } else { 0 } } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_BOOL,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 {
+                        boolVal: if value { -1 } else { 0 },
+                    },
+                },
             },
         })
     }
@@ -350,7 +404,15 @@ impl TryFrom<&PROPVARIANT> for bool {
 impl From<u8> for VARIANT {
     fn from(value: u8) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_UI1, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { bVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_UI1,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { bVal: value },
+                },
+            },
         })
     }
 }
@@ -359,7 +421,13 @@ impl From<u8> for PROPVARIANT {
     fn from(value: u8) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_UI1, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { bVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_UI1,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { bVal: value },
+                },
             },
         })
     }
@@ -370,7 +438,15 @@ impl From<u8> for PROPVARIANT {
 impl From<i8> for VARIANT {
     fn from(value: i8) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_I1, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { cVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_I1,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { cVal: value },
+                },
+            },
         })
     }
 }
@@ -379,7 +455,13 @@ impl From<i8> for PROPVARIANT {
     fn from(value: i8) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_I1, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { cVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_I1,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { cVal: value },
+                },
             },
         })
     }
@@ -390,7 +472,15 @@ impl From<i8> for PROPVARIANT {
 impl From<u16> for VARIANT {
     fn from(value: u16) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_UI2, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { uiVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_UI2,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { uiVal: value },
+                },
+            },
         })
     }
 }
@@ -399,7 +489,13 @@ impl From<u16> for PROPVARIANT {
     fn from(value: u16) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_UI2, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { uiVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_UI2,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { uiVal: value },
+                },
             },
         })
     }
@@ -426,7 +522,15 @@ impl TryFrom<&PROPVARIANT> for u16 {
 impl From<i16> for VARIANT {
     fn from(value: i16) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_I2, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { iVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_I2,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { iVal: value },
+                },
+            },
         })
     }
 }
@@ -435,7 +539,13 @@ impl From<i16> for PROPVARIANT {
     fn from(value: i16) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_I2, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { iVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_I2,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { iVal: value },
+                },
             },
         })
     }
@@ -462,7 +572,15 @@ impl TryFrom<&PROPVARIANT> for i16 {
 impl From<u32> for VARIANT {
     fn from(value: u32) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_UI4, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { ulVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_UI4,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { ulVal: value },
+                },
+            },
         })
     }
 }
@@ -471,7 +589,13 @@ impl From<u32> for PROPVARIANT {
     fn from(value: u32) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_UI4, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { ulVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_UI4,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { ulVal: value },
+                },
             },
         })
     }
@@ -498,7 +622,15 @@ impl TryFrom<&PROPVARIANT> for u32 {
 impl From<i32> for VARIANT {
     fn from(value: i32) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_I4, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { lVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_I4,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { lVal: value },
+                },
+            },
         })
     }
 }
@@ -507,7 +639,13 @@ impl From<i32> for PROPVARIANT {
     fn from(value: i32) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_I4, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { lVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_I4,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { lVal: value },
+                },
             },
         })
     }
@@ -534,7 +672,15 @@ impl TryFrom<&PROPVARIANT> for i32 {
 impl From<u64> for VARIANT {
     fn from(value: u64) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_UI8, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { ullVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_UI8,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { ullVal: value },
+                },
+            },
         })
     }
 }
@@ -543,7 +689,13 @@ impl From<u64> for PROPVARIANT {
     fn from(value: u64) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_UI8, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { uhVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_UI8,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { uhVal: value },
+                },
             },
         })
     }
@@ -570,7 +722,15 @@ impl TryFrom<&PROPVARIANT> for u64 {
 impl From<i64> for VARIANT {
     fn from(value: i64) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_I8, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { llVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_I8,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { llVal: value },
+                },
+            },
         })
     }
 }
@@ -579,7 +739,13 @@ impl From<i64> for PROPVARIANT {
     fn from(value: i64) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_I8, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { hVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_I8,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { hVal: value },
+                },
             },
         })
     }
@@ -606,7 +772,15 @@ impl TryFrom<&PROPVARIANT> for i64 {
 impl From<f32> for VARIANT {
     fn from(value: f32) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_R4, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { fltVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_R4,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { fltVal: value },
+                },
+            },
         })
     }
 }
@@ -615,7 +789,13 @@ impl From<f32> for PROPVARIANT {
     fn from(value: f32) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_R4, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { fltVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_R4,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { fltVal: value },
+                },
             },
         })
     }
@@ -626,7 +806,15 @@ impl From<f32> for PROPVARIANT {
 impl From<f64> for VARIANT {
     fn from(value: f64) -> Self {
         Self(imp::VARIANT {
-            Anonymous: imp::VARIANT_0 { Anonymous: imp::VARIANT_0_0 { vt: imp::VT_R8, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::VARIANT_0_0_0 { dblVal: value } } },
+            Anonymous: imp::VARIANT_0 {
+                Anonymous: imp::VARIANT_0_0 {
+                    vt: imp::VT_R8,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::VARIANT_0_0_0 { dblVal: value },
+                },
+            },
         })
     }
 }
@@ -635,7 +823,13 @@ impl From<f64> for PROPVARIANT {
     fn from(value: f64) -> Self {
         Self(imp::PROPVARIANT {
             Anonymous: imp::PROPVARIANT_0 {
-                Anonymous: imp::PROPVARIANT_0_0 { vt: imp::VT_R8, wReserved1: 0, wReserved2: 0, wReserved3: 0, Anonymous: imp::PROPVARIANT_0_0_0 { dblVal: value } },
+                Anonymous: imp::PROPVARIANT_0_0 {
+                    vt: imp::VT_R8,
+                    wReserved1: 0,
+                    wReserved2: 0,
+                    wReserved3: 0,
+                    Anonymous: imp::PROPVARIANT_0_0_0 { dblVal: value },
+                },
             },
         })
     }
