@@ -310,7 +310,7 @@
 /// Additionally a macro is generated by `generate!` (macros generating macros)
 /// called `export!`. The `export!` macro is given a component that implements
 /// the export `trait`s and then it will itself generate all necessary
-/// `#[no_mangle]` functions to implement the ABI required.
+/// `#[unsafe(no_mangle)]` functions to implement the ABI required.
 ///
 /// ## Exports: Multiple Interfaces
 ///
@@ -502,9 +502,9 @@
 /// Components are created by having exported WebAssembly functions with
 /// specific names, and these functions are not created when `generate!` is
 /// invoked. Instead these functions are created afterwards once you've defined
-/// your own type an implemented the various `trait`s for it. The `#[no_mangle]`
-/// functions that will become the component are created with the generated
-/// `export!` macro.
+/// your own type an implemented the various `trait`s for it. The
+/// `#[unsafe(no_mangle)]` functions that will become the component are created
+/// with the generated `export!` macro.
 ///
 /// Each call to `generate!` will itself generate a macro called `export!`.
 /// The macro's first argument is the name of a type that implements the traits
@@ -647,6 +647,12 @@
 ///
 ///     // Path to parse WIT and its dependencies from. Defaults to the `wit`
 ///     // folder adjacent to your `Cargo.toml`.
+///     //
+///     // This parameter also supports the form of a list, such as:
+///     // ["../path/to/wit1", "../path/to/wit2"]
+///     // Usually used in testing, our test suite may want to generate code
+///     // from wit files located in multiple paths within a single mod, and we
+///     // don't want to copy these files again.
 ///     path: "../path/to/wit",
 ///
 ///     // Enables passing "inline WIT". If specified this is the default
@@ -668,27 +674,43 @@
 ///     // By default this set is empty.
 ///     additional_derives: [PartialEq, Eq, Hash, Clone],
 ///
-///     // When generating bindings for imports it might be the case that
-///     // bindings were already generated in a different crate. For example
-///     // if your world refers to WASI types then the `wasi` crate already
-///     // has generated bindings for all WASI types and structures. In this
+///     // When generating bindings for interfaces that are not defined in the
+///     // same package as `world`, this option can be used to either generate
+///     // those bindings or point to already generated bindings.
+///     // For example, if your world refers to WASI types then the `wasi` crate
+///     // already has generated bindings for all WASI types and structures. In this
 ///     // situation the key `with` here can be used to use those types
 ///     // elsewhere rather than regenerating types.
+///     // If for example your world refers to some type and you want to use
+///     // your own custom implementation of that type then you can specify
+///     // that here as well. There is a requirement on the remapped (custom)
+///     // type to have the same internal structure and identical to what would
+///     // wit-bindgen generate (including alignment, etc.), since
+///     // lifting/lowering uses its fields directly.
 ///     //
-///     // The `with` key here only works for interfaces referred to by imported
-///     // functions. Additionally it only supports replacing types at the
-///     // interface level at this time.
+///     // If, however, your world refers to interfaces for which you don't have
+///     // already generated bindings then you can use the special `generate` value
+///     // to have those bindings generated.
 ///     //
-///     // When an interface is specified here no bindings will be generated at
-///     // all. It's assumed bindings are fully generated upstream. This is an
-///     // indicator that any further references to types defined in these
-///     // interfaces should use the upstream paths specified here instead.
+///     // The `with` key here works for interfaces and individual types.
+///     //
+///     // When an interface or type is specified here no bindings will be
+///     // generated at all. It's assumed bindings are fully generated
+///     // somewhere else. This is an indicator that any further references to types
+///     // defined in these interfaces should use the upstream paths specified
+///     // here instead.
 ///     //
 ///     // Any unused keys in this map are considered an error.
 ///     with: {
 ///         "wasi:io/poll": wasi::io::poll,
+///         "some:package/my-interface": generate,
+///         "some:package/my-interface/my-type": my_crate::types::MyType,
 ///     },
 ///
+///     // Indicates that all interfaces not present in `with` should be assumed
+///     // to be marked with `generate`.
+///     generate_all,
+///
 ///     // An optional list of function names to skip generating bindings for.
 ///     // This is only applicable to imports and the name specified is the name
 ///     // of the function.
@@ -782,9 +804,53 @@
 ///     // support the standard library itself depending on this crate one day.
 ///     std_feature,
 ///
-///     // Force a workaround to be emitted for pre-Rust-1.69.0 modules to
-///     // ensure that libc ctors run only once.
-///     run_ctors_once_workaround: true,
+///     // Disable a workaround to force wasm constructors to be run only once
+///     // when exported functions are called.
+///     disable_run_ctors_once_workaround: false,
+///
+///     // Whether to generate unused `record`, `enum`, `variant` types.
+///     // By default, they will not be generated unless they are used as input
+///     // or return value of a function.
+///     generate_unused_types: false,
+///
+///     // A list of "features" which correspond to WIT features to activate
+///     // when parsing WIT files. This enables `@unstable` annotations showing
+///     // up and having bindings generated for them.
+///     //
+///     // By default this is an empty list.
+///     features: ["foo", "bar", "baz"],
+///
+///     // Disables generation of a `#[used]` static to try harder to get the
+///     // custom section describing WIT types linked into the binary when
+///     // used in library-like situations. This is `false` by default with
+///     // `#[used]` statics being emitted.
+///     disable_custom_section_link_helpers: false,
+///
+///     // Write generated code to a .rs file, which allows the compiler to
+///     // emit more useful diagnostics for errors in the generated code.  This
+///     // is primarily useful for `wit-bindgen` developers.
+///     //
+///     // This does the same thing as setting `WIT_BINDGEN_DEBUG=1`, except
+///     // that it can be used on a more fine-grained basis (i.e. it only affects
+///     // the specific `generate!` call where it is used.
+///     debug: true,
+///
+///     // Generate async import and/or export bindings.
+///     //
+///     // The resulting bindings will use the component model
+///     // [async ABI](https://github.com/WebAssembly/component-model/blob/main/design/mvp/Async.md).
+///     // This may be specified either as a boolean (e.g. `async: true`, meaning
+///     // all imports and exports should use the async ABI) or as lists of
+///     // specific imports and/or exports as shown here:
+///     async: {
+///         imports: [
+///             "wasi:http/types@0.3.0-draft#[static]body.finish",
+///             "wasi:http/handler@0.3.0-draft#handle",
+///         ],
+///         exports: [
+///             "wasi:http/handler@0.3.0-draft#handle",
+///         ]
+///     }
 /// });
 /// ```
 ///
@@ -792,9 +858,10 @@
 #[cfg(feature = "macros")]
 pub use wit_bindgen_rust_macro::generate;
 
-// Re-export `bitflags` so that we can reference it from macros.
+// This re-export is no longer needed in new bindings and is only
+// here for compatibility.
 #[doc(hidden)]
-pub use bitflags;
+pub use rt::bitflags;
 
 mod pre_wit_bindgen_0_20_0;
 
@@ -803,13 +870,22 @@ pub mod examples;
 
 #[doc(hidden)]
 pub mod rt {
+    // Re-export `bitflags` so that we can reference it from macros.
+    pub use wit_bindgen_rt::bitflags;
+
+    #[cfg(target_arch = "wasm32")]
+    pub use wit_bindgen_rt::run_ctors_once;
+
     pub fn maybe_link_cabi_realloc() {
         #[cfg(feature = "realloc")]
         wit_bindgen_rt::maybe_link_cabi_realloc();
     }
 
-    #[cfg(feature = "realloc")]
+    #[cfg(all(feature = "realloc", not(target_env = "p2")))]
     pub use wit_bindgen_rt::cabi_realloc;
 
+    #[cfg(feature = "async")]
+    pub use wit_bindgen_rt::async_support;
+
     pub use crate::pre_wit_bindgen_0_20_0::*;
 }
